#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"bootp"
#define MODULE_DESCRIPTION	"bootp (rfc951) sniffer"
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	67
#define MODULE_IDENT_ID2	68

#define MODULE_FILTER		"udp and port 67 or port 68"

#define APSR_BOOTP_REQUEST	1
#define APSR_BOOTP_REPLY	2

/* TODO: BOOTP vendor specific extensions */

struct mod_bootp_header {
	unsigned int bootp_op:8;	/* packet opcode type */
	unsigned int bootp_htype:8;	/* hardware addr type */
	unsigned int bootp_hlen:8;	/* hardware addr length */
	unsigned int bootp_hops:8;	/* gateway hops */
	unsigned int bootp_xid;		/* transaction ID */
	unsigned short bootp_secs;	/* seconds since boot began */
	unsigned short bootp_flags;	/* flags: 0x8000 is broadcast */
	struct in_addr  bootp_ciaddr;	/* client IP address */
	struct in_addr  bootp_yiaddr;	/* 'your' IP address */
	struct in_addr  bootp_siaddr;	/* server IP address */
	struct in_addr  bootp_giaddr;	/* gateway IP address */
	unsigned char bootp_chaddr[16];	/* client hardware address */
	unsigned char bootp_sname[64];	/* server host name */
	unsigned char bootp_file[128];	/* boot file name */
	unsigned char bootp_vend[64];	/* vendor-specific area */
};


struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER;
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
					
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
	const struct mod_bootp_header *bootp_header;

 	bootp_header = (const struct mod_bootp_header *) buf;
	apsr_output("- BOOTP header -\n");

	apsr_output("  Opcode type: %u ", bootp_header->bootp_op);
	if(bootp_header->bootp_op == APSR_BOOTP_REQUEST)
 		apsr_output("(BOOTP request)\n");
	else if(bootp_header->bootp_op == APSR_BOOTP_REPLY)
		apsr_output("(BOOTP reply)\n");
	else
		apsr_output("(unknown BOOTP opcode!)\n");
	
	apsr_output("  Hardware type: %u ", bootp_header->bootp_htype);
	switch(bootp_header->bootp_htype) {
		case APSR_ARPHRD_NETROM:
		  apsr_output("(NETROM)\n");
		  break;
		case APSR_ARPHRD_ETHER:
		  apsr_output("(Ethernet 10/100Mbps)\n");
		  break;
		case APSR_ARPHRD_EETHER:
		  apsr_output("(Experimental Ethernet)\n");
		  break;
		case APSR_ARPHRD_AX25:
		  apsr_output("(AX.25 Level 2)\n");
		  break;
		case APSR_ARPHRD_PRONET:
		  apsr_output("(PROnet token ring)\n");
		  break;
		case APSR_ARPHRD_CHAOS:
		  apsr_output("(Chaosnet)\n");
		  break;
		case APSR_ARPHRD_IEEE802:
		  apsr_output("(IEEE 802.2 Ethernet/TR/TB)\n");
		  break;
		case APSR_ARPHRD_ARCNET:
		  apsr_output("(ARCNET)\n");
		  break;
		case APSR_ARPHRD_APPLETLK:
		  apsr_output("(Appletalk)\n");
		  break;
		case APSR_ARPHRD_DLCI:
		  apsr_output("(Frame Relay DLCI)\n");
		  break;
		case APSR_ARPHRD_ATM:
		  apsr_output("(ATM)\n");
		  break;
		case APSR_ARPHRD_METRICOM:
		  apsr_output("(Metricom STRIP)\n");
		  break;
		default:
		  apsr_output("(unknown type)\n");
		  break;
	};

	apsr_output("  Hardware length: %u\n", bootp_header->bootp_hlen);
	apsr_output("  Gateway hops: %u\n", bootp_header->bootp_hops);
	apsr_output("  Transaction ID: 0x%08x\n", apsr_ntohl(bootp_header->bootp_xid));
	apsr_output("  Seconds since boot began: %u\n", apsr_ntohs(bootp_header->bootp_secs));
	
	apsr_output("  Flags: %u ", apsr_ntohs(bootp_header->bootp_flags));
	if(bootp_header->bootp_flags == 0x8000)
		apsr_output("(broadcast)\n");
	else
		apsr_output("(unused)\n");

	if(bootp_header->bootp_op == APSR_BOOTP_REQUEST)
		apsr_output("  Client IP addr: %s\n", inet_ntoa(bootp_header->bootp_ciaddr));
	else if(bootp_header->bootp_op == APSR_BOOTP_REPLY)
		apsr_output("  Server IP addr: %s\n", inet_ntoa(bootp_header->bootp_siaddr));
		
	if(bootp_header->bootp_ciaddr.s_addr == 0)
		apsr_output("  Yclient IP addr: %s\n", inet_ntoa(bootp_header->bootp_yiaddr));

	apsr_output("  Gateway IP addr: %s\n", inet_ntoa(bootp_header->bootp_giaddr));
	apsr_output("  Client hardware addr: %s", ether2str((unsigned char *)bootp_header->bootp_chaddr));
	if(verbose)
		apsr_output(" (%s)\n", apsr_resolve_mac((unsigned char *)bootp_header->bootp_chaddr));
	else 
		apsr_output("\n");
	    
	if(*bootp_header->bootp_sname)
		apsr_output("\n  Server hostname: %s\n", bootp_header->bootp_sname);
	
	if(*bootp_header->bootp_file)
		apsr_output("  Boot file name: \"%s\"\n", bootp_header->bootp_file);
	
	if(*bootp_header->bootp_vend)
		apsr_output("  Vendor spec. exts: %s\n", "(not implemented)");
	
	#ifdef MODULE_DEBUG
	apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
		       family, protocol_number, id1, id2);
	#endif
	  
	return(RET_SUCCESS);
}

