#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.02"
#define MODULE_NAME		"dixie"
#define MODULE_DESCRIPTION	"Example DIXIE protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_PN2	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	96
#define MODULE_IDENT_ID2	96

#define MODULE_FILTER		"(udp or tcp) and port 96"

static const char *mod_dixie_returncodes[]= {
	" (Unknown)",
	" (Request Successfull)",
	" (Search failed)",
	" (Unknown Error occured)",
	" (Unknown DIXIE opcode)",
	" (Insufficient Access)",
	" (Malformed DN supplied)",
	" (Limit reached)",
	" (Modify before bind)",
	" (Fragment not found)",
	" (Attribute type invalid)",
	" (Attribute type does not exist)",
	" (Attribute value invalid)",
	" (Attribute value does not exist)",
};

#define APSR_DIXIE_OPCODE_READ		0x01
#define APSR_DIXIE_OPCODE_MODIFY	0x02
#define APSR_DIXIE_OPCODE_BIND		0x04
#define APSR_DIXIE_OPCODE_SEARCH	0x0F
#define APSR_DIXIE_OPCODE_LIST		0x10
#define APSR_DIXIE_OPCODE_ADD		0x11
#define APSR_DIXIE_OPCODE_REMOVE	0x12
#define APSR_DIXIE_OPCODE_MODIFY_RDN	0x13

#define APSR_DIXIE_RETURNCODE_SUCCESS	0x01
#define APSR_DIXIE_RETURNCODE_SEARCHFAIL 0x02
#define APSR_DIXIE_RETURNCODE_ERROR	0x03
#define APSR_DIXIE_RETURNCODE_OPWRONG	0x04
#define APSR_DIXIE_RETURNCODE_ACCESSDEN	0x05 
#define APSR_DIXIE_RETURNCODE_MALFORMDN	0x06
#define APSR_DIXIE_RETURNCODE_LIMITREACH	0x07
#define APSR_DIXIE_RETURNCODE_MODIFY 	0x08
#define APSR_DIXIE_RETURNCODE_FRAGREQ	0x09
#define APSR_DIXIE_RETURNCODE_ATTRTINV	0x0A
#define APSR_DIXIE_RETURNCODE_ATTRNOT	0x0B
#define APSR_DIXIE_RETURNCODE_ATTRVINV	0x0C
#define APSR_DIXIE_RETURNCODE_ATTRVEXI	0x0D

struct mod_dixie_req_header {
	unsigned char opcode;
	unsigned short identifier;
	unsigned int length;
	unsigned short unused;
	unsigned char options;
	unsigned char version;
	unsigned char search;
	unsigned short timelimit;
	unsigned short sizelimit;
};

struct mod_dixie_rep_header {
	unsigned char returncode;
	unsigned short identifier;
	unsigned int length;
	unsigned char unused1[3];
	unsigned char version;
	unsigned char unused2[5];
};
	
struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
	module_info.proto1      = MODULE_IDENT_PN1;
	module_info.proto2      = MODULE_IDENT_PN2;
	module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
					
	return(module_info);
}

extern char verbose;

/* Prototype */
void print_opcode(unsigned char);

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_dixie_req_header *dixie_req_header;
  struct mod_dixie_rep_header *dixie_rep_header;
	
  if(MODULE_IDENT_ID2 == id2) {
	    dixie_req_header = (struct mod_dixie_req_header *)buf;
	    apsr_output("- DIXIE Request Header -\n");
	    apsr_output("  Opcode: %u", dixie_req_header->opcode);
	    if(verbose) 
		    print_opcode(dixie_req_header->opcode);
	    apsr_output("\n  Identifier: %u, Length: %u, Unused: %u\n",
			    apsr_ntohs(dixie_req_header->identifier),
			    apsr_ntohl(dixie_req_header->length),
			    apsr_ntohs(dixie_req_header->unused));
	    apsr_output("  Options: 0x%02x, Version: %u, Search: %u\n",
			    dixie_req_header->options,
			    dixie_req_header->version,
			    dixie_req_header->search);
	    apsr_output("  Time Limit: %us, Size Limit: %u\n",
			    apsr_ntohs(dixie_req_header->timelimit),
			    apsr_ntohs(dixie_req_header->sizelimit));
  }
  else if(MODULE_IDENT_ID1 == id1) {
            dixie_rep_header = (struct mod_dixie_rep_header *)buf;
	    apsr_output("- DIXIE Reply Header -\n");
	    apsr_output("  Returncode: %u", dixie_rep_header->returncode);
	    if(verbose)
		    apsr_output("%s", mod_dixie_returncodes[dixie_rep_header->returncode]);
	    apsr_output("\n  Identifier: %u, Length: %u, Version: %u\n",
			    apsr_ntohs(dixie_rep_header->identifier),
			    apsr_ntohl(dixie_rep_header->length),
			    dixie_rep_header->version);
  }	 

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}

void print_opcode(unsigned char opcode) {
	switch(opcode) {
		case APSR_DIXIE_OPCODE_READ:
			apsr_output(" (Read)");
			break;
		case APSR_DIXIE_OPCODE_MODIFY:
			apsr_output(" (Modify)");
			break;
		case APSR_DIXIE_OPCODE_BIND:
			apsr_output(" (Bind)");
			break;
		case APSR_DIXIE_OPCODE_SEARCH:
			apsr_output(" (Search)");
			break;
		case APSR_DIXIE_OPCODE_LIST:
			apsr_output(" (List)");
			break;
		case APSR_DIXIE_OPCODE_ADD:
			apsr_output(" (Add)");
			break;
		case APSR_DIXIE_OPCODE_REMOVE:
			apsr_output(" (Remove)");
			break;
		case APSR_DIXIE_OPCODE_MODIFY_RDN:
			apsr_output(" (Modify RDN)");
			break;
		default:
			apsr_output(" (Unknown)");
			break;
	}
}
