#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.03"
#define MODULE_NAME		"hsrp"
#define MODULE_DESCRIPTION	"Example HSRP (Cisco Hot Standby Router Protocol) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	1985	
#define MODULE_IDENT_ID2	1985

#define MODULE_FILTER		"udp and port 1985"

#define APSR_HSRP_STATE_INITIAL		0	/* rfc 2281 */
#define APSR_HSRP_STATE_LEARN		1       /* rfc 2281 */
#define APSR_HSRP_STATE_LISTEN		2       /* rfc 2281 */
#define APSR_HSRP_STATE_SPEAK		4       /* rfc 2281 */
#define APSR_HSRP_STATE_STANDBY		8       /* rfc 2281 */
#define APSR_HSRP_STATE_ACTIVE		16      /* rfc 2281 */

static const char *mod_hsrp_opcode[]= {
	" (Hello)",
	" (Coup)",
	" (Resign)",
};

struct mod_hsrp_header {
	unsigned char version;
	unsigned char opcode;
	unsigned char state;
	unsigned char hellotime;
	unsigned char holdtime;
	unsigned char priority;
	unsigned char group;
	unsigned char reserved;
	unsigned char auth_data[8];
	struct in_addr virtual_ip;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_hsrp_header *hsrp_header;
  char *virt_ip;
  unsigned short i;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    hsrp_header = (struct mod_hsrp_header *)buf;
	    virt_ip = strdup(inet_ntoa(hsrp_header->virtual_ip));
	    apsr_output("- HSRP Header -\n");
	    apsr_output("  Version: %u\n", hsrp_header->version);
	    apsr_output("  Opcode: %u", hsrp_header->opcode);
	    if(verbose)
		    apsr_output("%s", mod_hsrp_opcode[hsrp_header->opcode]);
	    apsr_output("\n  State: %u", hsrp_header->state);
	    if(verbose) {
		    switch(hsrp_header->state) {
			    case APSR_HSRP_STATE_INITIAL:
				    apsr_output(" (Initial)");
				    break;
		            case APSR_HSRP_STATE_LEARN:
				    apsr_output(" (Learn)");
				    break;
			    case APSR_HSRP_STATE_LISTEN:
				    apsr_output(" (Listen)");
				    break;
			    case APSR_HSRP_STATE_SPEAK:
				    apsr_output(" (Speak)");
				    break;
			    case APSR_HSRP_STATE_STANDBY:
				    apsr_output(" (Standby)");
				    break;
			    case APSR_HSRP_STATE_ACTIVE:
				    apsr_output(" (Active)");
				    break;
			    default:
				    apsr_output(" (unknown)");
				    break;
		    }
	    }		    
	    apsr_output("\n  Hello time: %us, Hold time: %us\n",
			    hsrp_header->hellotime, hsrp_header->holdtime);
	    apsr_output("  Priority: %u, Group: %u\n",
			    hsrp_header->priority, hsrp_header->group);
	    apsr_output("  Authentication Data: \"");
	    for(i = 0; i <= 7; i++) {
		    if(isprint(hsrp_header->auth_data[i])) 
			    apsr_output("%c", hsrp_header->auth_data[i]);
		    else 
			    apsr_output(".");
	    }
	    apsr_output("\"");
	    if(verbose) {
		    if(hsrp_header->auth_data[0] == 0x63 &&
		       hsrp_header->auth_data[1] == 0x69 &&
		       hsrp_header->auth_data[2] == 0x73 &&
		       hsrp_header->auth_data[3] == 0x63 &&
		       hsrp_header->auth_data[4] == 0x6f &&
		       hsrp_header->auth_data[5] == 0x00 &&
		       hsrp_header->auth_data[6] == 0x00 &&
		       hsrp_header->auth_data[7] == 0x00)
		      apsr_output(" (Default password)");
            }		    
	    apsr_output("\n  Virtual IP: %s\n", virt_ip);
	    free(virt_ip);
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
